/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/


#include "mxsmbus.h"
#include "utils.h"

#ifdef __cplusplus    // If used by C++ code, 
extern "C" {          // we need to export the C interface
#endif

#define CMD_CODE_SET_PHY_REG_ADDR 0x0 //00000000
#define CMD_CODE_GET_PHY_REG_ADDR 0x1 //00000001
#define CMD_CODE_WRITE_REG_DATA 0x3 //00000011
#define CMD_CODE_READ_REG_DATA 0x4 //00000100

#define DATACOUNT_READ_WRITE_MDIO_REG_DATA 2
#define DATACOUNT_PHY_REG_ADDR 2

#define PHY_ADDR_LANA 0x4
#define PHY_ADDR_LANB 0x6
#define PHY_ADDR_INTERLINK 0x7

#define REG_ADDR_SPARE_CONTROL 0x1C
#define REG_ADDR_PHY_EXTENDED_CONTROL 0x10

typedef struct _BIT_PHY_SPARE_CONTROL_1
{
	unsigned short link_led_mode : 1; //bit0
	unsigned short reserved1 : 1; //bit1
	unsigned short link_speed_led_mode : 1; //bit2
	unsigned short reserved3 : 1; //bit3 
	unsigned short reserved4 : 1; //bit4
	unsigned short reserved5 : 1; //bit5
	unsigned short reserved6 : 1; //bit6
	unsigned short reserved7 : 1; //bit7
	unsigned short reserved8 : 1; //bit8
	unsigned short reserved9 : 1; //bit9
	unsigned short shadow_register_selector : 5; //bit10~14
	unsigned short write_enable : 1; //bit15

}BIT_PHY_SPARE_CONTROL_1, *PBIT_PHY_SPARE_CONTROL_1;

typedef union _PHY_SPARE_CONTROL_1
{
	unsigned short regData;
	BIT_PHY_SPARE_CONTROL_1 bitInfo;

}PHY_SPARE_CONTROL_1, *PPHY_SPARE_CONTROL_1;

typedef struct _BIT_MODE_CONTROL
{
	unsigned short enable_sgmii_reg : 1; //bit0
	unsigned short mode_select : 2; //bit1, bit2
	unsigned short reserved3 : 1; //bit3 
	unsigned short reserved4 : 1; //bit4
	unsigned short copper_energy_detect : 1; //bit5
	unsigned short sgmii_link : 1; //bit6
	unsigned short copper_link : 1; //bit7
	unsigned short mode_select_change : 1; //bit8
	unsigned short reserved9 : 1; //bit9
	unsigned short shadow_register_selector : 5; //bit10~14
	unsigned short write_enable : 1; //bit15

}BIT_MODE_CONTROL, *PBIT_MODE_CONTROL;

typedef union _MODE_CONTROL
{
	unsigned short regData;
	BIT_MODE_CONTROL bitInfo;

}MODE_CONTROL, *PMODE_CONTROL;

typedef struct _BIT_PHY_EXTENDED_CONTROL
{
	unsigned short reserved0 : 1; //bit0
	unsigned short reserved1 : 1; //bit1
	unsigned short reserved2 : 1; //bit2
	unsigned short force_leds_off : 1; //bit3 
	unsigned short force_leds_on : 1; //bit4
	unsigned short enable_led_traffic_mode : 1; //bit5
	unsigned short reset_scrambler : 1; //bit6
	unsigned short bypass_receive_symbol : 1; //bit7
	unsigned short bypass_nzri_encoder : 1; //bit8
	unsigned short bypass_scrambler : 1; //bit9
	unsigned short bypass_4b_encoder : 1; //bit10
	unsigned short force_interrupt : 1; //bit11
	unsigned short interrupt_disable : 1; //bit12
	unsigned short transmit_disable : 1; //bit13
	unsigned short disable_auto_mdi_crossover : 1; //bit14
	unsigned short reserved15 : 1; //bit15

}BIT_PHY_EXTENDED_CONTROL, *PBIT_PHY_EXTENDED_CONTROL;

typedef union _PHY_EXTENDED_CONTROL
{
	unsigned short regData;
	BIT_PHY_EXTENDED_CONTROL bitInfo;

}PHY_EXTENDED_CONTROL, *PPHY_EXTENDED_CONTROL;

/*
*
* Main function to set Link Speed mode
*
*/
//int SetLinkSpeedMode(UINT8 slaveAddr);
int SetActivityMode(UINT8 slaveAddr);
int ConfigInterLinkPortLinkMode(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short regData);
int ConfigLanAPortLinkMode(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short regData);
int GetLanAPortSetting(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short *regData);
int ConfigLanBPortLinkMode(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short regData);
int GetLanBPortSetting(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short *regData);

/*
*
* MDIO Access Functions
*
*/
int ConfigModeControl(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short regData);
int GetModeControl(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short *regData);
int ConfigActivityLedMode(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short regData);
int GetActivityLedMode(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short *regData);
int ConfigLinkLedMode(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short regData);
int GetLinkLedMode(UINT8 slaveAddr, UINT8 phyAddr, UINT8 regAddr, unsigned short *regData);
int SetMdioPhyRegAddr(UINT8 slaveAddr, UINT8 *mdioPhyRegAddr);
int GetMdioPhyRegAddr(UINT8 slaveAddr, UINT8 *mdioPhyRegAddr);
int WriteMdioRegData(UINT8 slaveAddr, UINT8 *mdioRegData);
int ReadMdioRegData(UINT8 slaveAddr, UINT8 *mdioRegData);

#ifdef __cplusplus    // If used by C++ code, 
}
#endif